@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns, FlexibleInstances, CPP #-}
+{-# LANGUAGE BangPatterns, CPP #-}
 
 -- | File descriptor cache to avoid locks in kernel.
 
@@ -23,14 +23,13 @@ module Network.Wai.Handler.Warp.FdCache (
   ) where
 
 import Control.Applicative ((<$>), (<*>))
-import Control.Concurrent (threadDelay)
-import Control.Exception (bracket, mask_)
+import Control.Exception (bracket)
 import Data.Hashable (hash)
 import Network.Wai.Handler.Warp.IORef
 import Network.Wai.Handler.Warp.MultiMap
-import Network.Wai.Handler.Warp.Thread
 import System.Posix.IO (openFd, OpenFileFlags(..), defaultFileFlags, OpenMode(ReadOnly), closeFd)
 import System.Posix.Types (Fd)
+import Control.Reaper
 
 ----------------------------------------------------------------
 
@@ -68,19 +67,10 @@ type Hash = Int
 type FdCache = MMap Hash FdEntry
 
 -- | Mutable Fd cacher.
-newtype MutableFdCache = MutableFdCache (IORef FdCache)
+type MutableFdCache = Reaper FdCache (Hash, FdEntry)
 
 fdCache :: MutableFdCache -> IO FdCache
-fdCache (MutableFdCache ref) = readIORef ref
-
-swapWithNew :: IORef FdCache -> IO FdCache
-swapWithNew ref = atomicModifyIORef' ref $ \t -> (empty, t)
-
-update :: MutableFdCache -> (FdCache -> FdCache) -> IO ()
-update (MutableFdCache ref) = update' ref
-
-update' :: IORef FdCache -> (FdCache -> FdCache) -> IO ()
-update' ref f = atomicModifyIORef' ref $ \t -> (f t, ())
+fdCache mfc = reaperRead mfc
 
 look :: MutableFdCache -> FilePath -> Hash -> IO (Maybe FdEntry)
 look mfc path key = searchWith key check <$> fdCache mfc
@@ -106,17 +96,18 @@ withFdCache duration action = bracket (initialize duration)
 -- The first argument is a cache duration in second.
 initialize :: Int -> IO (Maybe MutableFdCache)
 initialize 0 = return Nothing
-initialize duration = do
-    ref' <- forkIOwithBreakableForever empty $ \ref -> do
-        threadDelay duration
-        clean ref
-    return (Just (MutableFdCache ref'))
+initialize duration = Just <$> mkReaper defaultReaperSettings
+    { reaperAction = clean
+    , reaperDelay = duration
+    , reaperCons = uncurry insert
+    , reaperNull = isEmpty
+    , reaperEmpty = empty
+    }
 
-clean :: IORef FdCache -> IO ()
-clean ref = do
-    old <- swapWithNew ref
+clean :: FdCache -> IO (FdCache -> FdCache)
+clean old = do
     new <- pruneWith old prune
-    update' ref (merge new)
+    return $ merge new
 
 prune :: t -> Some FdEntry -> IO [(t, Some FdEntry)]
 prune k v@(One (FdEntry _ fd mst)) = status mst >>= prune'
@@ -138,8 +129,8 @@ prune k (Tom ent@(FdEntry _ fd mst) vs) = status mst >>= prune'
 
 terminate :: Maybe MutableFdCache -> IO ()
 terminate Nothing = return ()
-terminate (Just (MutableFdCache ref)) = mask_ $ do
-    !t <- breakForever ref
+terminate (Just mfc) = do
+    !t <- reaperStop mfc
     mapM_ closeIt $ toList t
   where
     closeIt (_, FdEntry _ fd _) = closeFd fd
@@ -153,7 +144,7 @@ getFd mfc path = look mfc path key >>= getFd'
     key = hash path
     getFd' Nothing = do
         ent@(FdEntry _ fd mst) <- newFdEntry path
-        update mfc (insert key ent)
+        reaperAdd mfc (key, ent)
         return (fd, refresh mst)
     getFd' (Just (FdEntry _ fd mst)) = do
         refresh mst
